'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var helpers = require('./helpers.js');
var update = require('./update.js');
var css = require('./css.js');
var render = require('./render.js');
var mergeSlots = require('./mergeSlots.js');

function serialise(node, state) {
  let ds = helpers.getDataScript(node) || helpers.createDataScript(node);
  ds.innerText = JSON.stringify(state);
}

const define = (name, factory, template, css$1) => {
  if (customElements.get(name)) return

  if (css$1) css.appendStyles(name, css$1);

  customElements.define(
    name,
    class extends HTMLElement {
      async connectedCallback() {
        if (!this.initialised) {
          let config = factory(this);

          if (config instanceof Promise) config = await config;

          let { subscribe, shadow, observe = [] } = config;

          this.connectedCallback = config.connectedCallback;
          this.disconnectedCallback = config.disconnectedCallback;

          const ds = helpers.getDataScript(this);

          const { dispatch, getState, onChange, updated, refs } = update.configure(
            {
              ...config,
              state: ds ? JSON.parse(ds.innerText) : config.state,
            });

          this.store = { dispatch, getState };

          let state = getState();

          let observedAttributes = observe.map(helpers.pascalToKebab);

          let sa = this.setAttribute;
          this.setAttribute = (k, v) => {
            if (observedAttributes.includes(k)) {
              let { name, value } = helpers.attributeToProp(k, v);

              dispatch({
                type: "SET",
                payload: { name, value },
              });
            }
            sa.apply(this, [k, v]);
          };
          let ra = this.removeAttribute;
          this.removeAttribute = (k) => {
            if (observedAttributes.includes(k)) {
              let { name, value } = helpers.attributeToProp(k, null);
              dispatch({
                type: "SET",
                payload: { name, value },
              });
            }
            ra.apply(this, [k]);
          };

          observedAttributes.forEach((name) => {
            let property = helpers.attributeToProp(name).name;
            let value;

            if (this.hasAttribute(name)) {
              value = this.getAttribute(name);
            } else {
              value = this[property] || state[property];
            }

            Object.defineProperty(this, property, {
              get() {
                return getState()[property]
              },
              set(v) {
                dispatch({
                  type: "SET",
                  payload: { name: property, value: v },
                });
                if (helpers.isPrimitive(v)) {
                  helpers.applyAttribute(this, property, v);
                }
              },
            });

            this[property] = value;
          });

          let beforeMountCallback;

          if (shadow) {
            this.attachShadow({
              mode: shadow,
            });
          } else {
            beforeMountCallback = (frag) => mergeSlots.mergeSlots(this, frag);
          }

          onChange(
            render.render(
              this.shadowRoot || this,
              { getState, dispatch, refs },
              template,
              () => {
                const state = getState();

                serialise(this, state);
                observe.forEach((k) => {
                  let v = state[k];
                  if (helpers.isPrimitive(v)) helpers.applyAttribute(this, k, v);
                });
                subscribe?.(getState());
                updated();
              },
              beforeMountCallback
            )
          );

          if (!ds) serialise(this, getState());
        }

        if (!this._C) this.connectedCallback?.(this.store);
        this._C = false;
      }
      disconnectedCallback() {
        this._C = true;
        requestAnimationFrame(() => {
          if (this._C) this.disconnectedCallback?.();
          this._C = false;
        });
      }
    }
  );
};

exports.define = define;
