'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var helpers = require('./helpers.js');

const VALUE = 1;
const KEY = 2;
const FUNCTION = 3;

const hasMustache = (v) => v.match(/({{[^{}]+}})/);

const getParts = (value) =>
  value
    .trim()
    .split(/({{[^{}]+}})/)
    .filter((v) => v)
    .map((value) => {
      let match = value.match(/{{([^{}]+)}}/);

      if (!match)
        return {
          type: VALUE,
          value,
          negated: false,
        }

      value = match[1].trim();
      let negated = value.charAt(0) === "!";
      if (negated) value = value.slice(1);

      return {
        type: KEY,
        value,
        negated,
      }
    });

const getValueFromParts = (target, parts) => {
  return parts.reduce((a, part) => {
    let { type, value, negated } = part;

    let v;

    if (type === VALUE) v = value;
    if (type === KEY) {
      v = helpers.getValueAtPath(value, target);
    }
    if (type === FUNCTION) {
      let args = part.args.map((value) => helpers.getValueAtPath(value, target));

      v = helpers.getValueAtPath(part.method, target)?.(...args);
    }

    if (negated) v = !v;

    return a || a === 0 ? a + v : v
  }, "")
};

exports.getParts = getParts;
exports.getValueFromParts = getValueFromParts;
exports.hasMustache = hasMustache;
